@@ -41,6 +41,13 @@ from .generator import BaseGenerator, BuildGenerator
 __all__ = ("main", "generate")
 
 
+DEFAULT_RULES = {
+    "error": {
+        "command": "echo '${message}'; exit -1",
+    }
+}
+
+
 def generate(root: str):
     """Parse ``reprebuild.yaml`` files and write a ``build.ninja`` file.
 
@@ -66,6 +73,7 @@ def generate(root: str):
 
         # Write all rules, even if some are not used.
         rules = _collect_dicts(generators, "rules")
+        rules.update(DEFAULT_RULES)
         writer.comment("All rules (except generator)")
         for rule_name, rule in rules.items():
             writer.rule(name=rule_name, **rule)
@@ -79,31 +87,48 @@ def generate(root: str):
         # Write all build lines with comments and defaults
         outputs = set()
         defaults = set()
-        not_scanned = set()
+        gendeps = set()
         for generator in generators:
-            for records, new_not_scanned in generator(outputs, defaults):
-                not_scanned.update(new_not_scanned)
+            for records, new_gendeps in generator(outputs, defaults):
+                gendeps.update(new_gendeps)
                 for record in records:
                     if isinstance(record, str):
                         writer.comment(record)
                     elif isinstance(record, list):
-                        writer.default(record)
-                        defaults.update(record)
+                        for default in record:
+                            if default not in defaults:
+                                writer.default(record)
+                                defaults.update(record)
                     elif isinstance(record, dict):
-                        writer.build(**record)
-                        outputs.update(record["outputs"])
-                        outputs.update(record.get("implicit_outputs", []))
+                        new_outputs = set(record["outputs"])
+                        new_outputs |= set(record.get("implicit_outputs", []))
+                        if outputs.isdisjoint(new_outputs):
+                            writer.build(**record)
+                            outputs.update(new_outputs)
+                        else:
+                            # Some outputs are repeated, in which case no new build lines are
+                            # written. It is assumed that the preceding builds are more specific
+                            # and therefore should take priority over later ones.
+                            # To maintain sanity, ambiguous cases are not allowed, i.e. all
+                            # new outputs should already exist.
+                            if not new_outputs.issubset(outputs):
+                                raise ValueError(
+                                    "Outputs are partially generated in previous builds. "
+                                    f"New: {new_outputs - outputs} "
+                                    f"Existing: {new_outputs & outputs}"
+                                )
+                            writer.comment("Skipping due to overlap with previous builds.")
                     else:
                         raise TypeError("Cannot process ")
                 writer.newline()
 
         # Insert generator if some files could not be scanned
-        if len(not_scanned) > 0:
+        if len(gendeps) > 0:
             writer.newline()
-            writer.comment("Some dependencies were absent, which could induce additional builds.")
-            writer.comment("This means the build.ninja file needs to be regenerated.")
+            writer.comment("Some files influence the generation of the build files.")
+            writer.comment("When they change, the build.ninja file must be regenerated.")
             writer.rule("generator", command="rr-generator .", generator=True)
-            writer.build(rule="generator", implicit=sorted(not_scanned), outputs="build.ninja")
+            writer.build(rule="generator", implicit=sorted(gendeps), outputs="build.ninja")
 
 
 def _collect_dicts(generators: list[BaseGenerator], attr_name: str) -> dict[str:object]:
