@@ -21,8 +21,9 @@ from .check_hrefs import check_hrefs
 from .latex import latex, latex_diff, latex_flat
 from .mupdf import pdf_add_notes, pdf_merge, pdf_nup
 from .python_script import python_script
+from .shell_script import shell_script
 from .transform import convert_odf_pdf, convert_svg_pdf, copy, markdown_pdf, pdf_raster, render
-from .zip import repro_zip, repro_zip_latex
+from .zip import zip_latex, zip_manifest, zip_plain
 
 __all__ = ("get_commands",)
 
@@ -43,6 +44,8 @@ def get_commands():
         markdown_pdf,
         pdf_raster,
         render,
-        repro_zip,
-        repro_zip_latex,
+        shell_script,
+        zip_latex,
+        zip_manifest,
+        zip_plain,
     ]
