@@ -22,11 +22,13 @@
 import os
 import subprocess
 import time
+import traceback
+from glob import glob
 
 from watchdog.events import FileSystemEventHandler
 from watchdog.observers import Observer
 
-from .__main__ import DEFAULT_PATTERNS, DEFAULT_RULES, parse_args, write_ninja
+from .__main__ import generate, parse_args, sanity_check
 
 
 class AnyChangeHandler(FileSystemEventHandler):
@@ -71,10 +73,13 @@ class AnyChangeHandler(FileSystemEventHandler):
 
 def main():
     """Main program."""
+    sanity_check()
     args = parse_args()
     observer = Observer()
     event_handler = AnyChangeHandler()
-    observer.schedule(event_handler, ".", True)
+    observer.schedule(event_handler, ".")
+    for path in glob("*/"):
+        observer.schedule(event_handler, path)
     observer.start()
     try:
         while True:
@@ -82,11 +87,18 @@ def main():
                 time.sleep(1)
                 continue
             event_handler.snooze()
-            write_ninja(DEFAULT_PATTERNS, DEFAULT_RULES)
-            subprocess.run(["ninja"] + args)
+            try:
+                generate(os.getcwd())
+                subprocess.run(
+                    ["ninja", *args],
+                    check=False,
+                    env=os.environ | {"NINJA_STATUS": "\033[1;36;40m[%f/%t]\033[0;0m "},
+                )
+            except Exception:
+                print(traceback.format_exc())
             time.sleep(0.1)
             event_handler.reset()
-            print("  Waiting for new changes.")
+            print("\033[1;36;40m-- Waiting for new changes. --\033[0;0m")
     except KeyboardInterrupt:
         print("  See you!")
     finally:
