@@ -20,11 +20,10 @@
 """Unit tests for reprepbuild.builtin.latex and reprepbuild.scripts.latex"""
 
 import contextlib
-import io
 import os
 
 import pytest
-from reprepbuild.builtin.latex import latex, latex_diff, latex_flat
+from reprepbuild.builtin.latex import latex, latex_diff, latex_flat, scan_latex_deps
 from reprepbuild.scripts.latex import (
     DEFAULT_MESSAGE,
     MESSAGE_SUFFIX,
@@ -33,12 +32,22 @@ from reprepbuild.scripts.latex import (
     parse_latex_log,
 )
 
+
+@contextlib.contextmanager
+def local_file(contents, filename, tmpdir):
+    """Change to a temporary directory and create a file with given contents."""
+    with contextlib.chdir(tmpdir):
+        with open(filename, "w") as fh:
+            fh.write(contents)
+        yield
+
+
 BUILDS_LATEX = [
     {
         "rule": "latex",
         "inputs": ["sub/main.tex"],
         "outputs": ["sub/main.pdf"],
-        "implicit_outputs": ["sub/main.log", "sub/main.aux", "sub/main.out", "sub/main.fls"],
+        "implicit_outputs": ["sub/main.log", "sub/main.aux", "sub/main.fls"],
         "implicit": [],
         "variables": {"latex": "pdflatex"},
     }
@@ -46,8 +55,8 @@ BUILDS_LATEX = [
 
 
 def test_write_build_latex():
-    builds, not_scanned = latex.generate(["sub/main.tex"], [], None, {})
-    assert not_scanned == ["sub/main.tex"]
+    builds, gendeps = latex.generate(["sub/main.tex"], [], None, {})
+    assert gendeps == ["sub/main.tex"]
     assert BUILDS_LATEX == builds
 
 
@@ -75,7 +84,6 @@ BUILDS_LATEX_BIBTEX1 = [
             "main.bbl",
             "main.log",
             "main.aux",
-            "main.out",
             "main.fls",
         ],
         "implicit": ["smile.pdf", "sub/foo.tex", "table.tex", "references.bib"],
@@ -94,11 +102,39 @@ def test_write_build_latex_bibtex1(tmpdir):
     with open(os.path.join(tmpdir, "main.tex"), "w") as fh:
         fh.write(MAIN1_TEX)
     with contextlib.chdir(tmpdir):
-        builds, not_scanned = latex.generate(["main.tex"], [], None, {})
-    assert not_scanned == ["sub/foo.tex", "table.tex"]
+        builds, gendeps = latex.generate(["main.tex"], [], None, {})
+    assert gendeps == ["main.tex", "sub/foo.tex", "table.tex"]
     assert BUILDS_LATEX_BIBTEX1 == builds
 
 
+BUILDS_LATEX1 = [
+    {
+        "rule": "latex",
+        "inputs": ["main.tex"],
+        "outputs": ["main.pdf"],
+        "implicit_outputs": [
+            "main.log",
+            "main.aux",
+            "main.fls",
+        ],
+        "implicit": ["smile.pdf", "sub/foo.tex", "table.tex", "main.bbl"],
+        "variables": {
+            "latex": "pdflatex",
+        },
+    }
+]
+
+
+def test_write_build_latex1(tmpdir):
+    tmpdir = str(tmpdir)
+    with open(os.path.join(tmpdir, "main.tex"), "w") as fh:
+        fh.write(MAIN1_TEX)
+    with contextlib.chdir(tmpdir):
+        builds, gendeps = latex.generate(["main.tex"], [], {"skip_bibtex": True}, {})
+    assert gendeps == ["main.tex", "sub/foo.tex", "table.tex"]
+    assert BUILDS_LATEX1 == builds
+
+
 SUB1_FOO_TEX = r"""
 Here is some text.
 \includegraphics{plot.pdf}
@@ -115,7 +151,6 @@ BUILDS_LATEX_BIBTEX_FOO1 = [
             "main.bbl",
             "main.log",
             "main.aux",
-            "main.out",
             "main.fls",
         ],
         "implicit": ["smile.pdf", "sub/foo.tex", "sub/plot.pdf", "table.tex", "references.bib"],
@@ -138,8 +173,8 @@ def test_write_build_latex_bibtex_foo1(tmpdir):
     with open(os.path.join(subdir, "foo.tex"), "w") as fh:
         fh.write(SUB1_FOO_TEX)
     with contextlib.chdir(tmpdir):
-        builds, not_scanned = latex.generate(["main.tex"], [], None, {})
-    assert not_scanned == ["table.tex"]
+        builds, gendeps = latex.generate(["main.tex"], [], None, {})
+    assert gendeps == ["main.tex", "sub/foo.tex", "table.tex"]
     assert BUILDS_LATEX_BIBTEX_FOO1 == builds
 
 
@@ -170,7 +205,6 @@ BUILDS_LATEX_BIBTEX_TABLE2 = [
             "sub/main.bbl",
             "sub/main.log",
             "sub/main.aux",
-            "sub/main.out",
             "sub/main.fls",
         ],
         "implicit": ["sub/smile.pdf", "sub/table.tex", "sub/references.bib"],
@@ -193,8 +227,8 @@ def test_write_build_latex_bibtex_table2(tmpdir):
     with open(os.path.join(subdir, "table.tex"), "w") as fh:
         fh.write(TABLE2_TEX)
     with contextlib.chdir(tmpdir):
-        builds, not_scanned = latex.generate(["sub/main.tex"], [], None, {})
-    assert not_scanned == []
+        builds, gendeps = latex.generate(["sub/main.tex"], [], None, {})
+    assert gendeps == ["sub/main.tex", "sub/table.tex"]
     assert BUILDS_LATEX_BIBTEX_TABLE2 == builds
 
 
@@ -209,8 +243,8 @@ BUILDS_LATEX_FLAT = [
 
 
 def test_write_build_latex_flat():
-    builds, not_scanned = latex_flat.generate(["sub/main.tex"], ["sub/main-flat.tex"], None, {})
-    assert not_scanned == ["sub/main.tex"]
+    builds, gendeps = latex_flat.generate(["sub/main.tex"], ["sub/main-flat.tex"], None, {})
+    assert gendeps == ["sub/main.tex"]
     assert BUILDS_LATEX_FLAT == builds
 
 
@@ -239,10 +273,10 @@ BUILDS_LATEX_DIFF = [
 
 
 def test_write_build_latex_diff():
-    builds, not_scanned = latex_diff.generate(
+    builds, gendeps = latex_diff.generate(
         ["sub/main.tex", "sub/old/main.tex"], ["sub/main-diff.tex"], None, {}
     )
-    assert not_scanned == []
+    assert gendeps == []
     assert BUILDS_LATEX_DIFF == builds
 
 
@@ -275,13 +309,48 @@ I'm skipping whatever remains of this entry
 """
 
 
-def test_parse_bibtex_log1():
-    error_info = parse_bibtex_log(io.StringIO(BIBTEX_BLG1))
+def test_parse_bibtex_log1(tmpdir):
+    with local_file(BIBTEX_BLG1, "bibtex.blg", tmpdir):
+        error_info = parse_bibtex_log("bibtex.blg")
     assert error_info.program == "BibTeX"
     assert error_info.src == "references.bib"
     assert error_info.message == BIBTEX_BLG1_MESSAGE.strip()
 
 
+BIBTEX_BLG2 = """\
+This is BibTeX, Version 0.99d (TeX Live 2022/CVE-2023-32700 patched)
+There are not obvious problems in the log file.
+"""
+
+
+def test_parse_bibtex_log2(tmpdir):
+    with local_file(BIBTEX_BLG2, "bibtex.blg", tmpdir):
+        error_info = parse_bibtex_log("bibtex.blg")
+    assert error_info.program == "BibTeX"
+    assert error_info.src == "(could not detect source file)"
+    assert error_info.message == DEFAULT_MESSAGE.format(path="bibtex.blg")
+
+
+BIBTEX_BLG3 = r"""\
+This is BibTeX, Version 0.99d (TeX Live 2023)
+Capacity: max_strings=200000, hash_size=200000, hash_prime=170003
+The top-level auxiliary file: reply.aux
+I found no \bibstyle command---while reading file reply.aux
+You've used 15 entries,
+            0 wiz_defined-function locations,
+            114 strings with 812 characters,
+and the built_in function-call counts, 0 in all, are:
+"""
+
+
+def test_parse_bibtex_log3(tmpdir):
+    with local_file(BIBTEX_BLG3, "bibtex.blg", tmpdir):
+        error_info = parse_bibtex_log("bibtex.blg")
+    assert error_info.program == "BibTeX"
+    assert error_info.src == "reply.aux"
+    assert error_info.message == r"I found no \bibstyle command---while reading file reply.aux"
+
+
 LATEX_LOG1 = r"""
 **article
 (./article.tex
@@ -314,12 +383,16 @@ l.396         \begin{center}\foo
 """
 
 
-def test_parse_latex_log1():
-    rebuild, error_info = parse_latex_log(io.StringIO(LATEX_LOG1))
+def test_parse_latex_log1(tmpdir):
+    with local_file(LATEX_LOG1, "article.log", tmpdir):
+        rebuild, error_info = parse_latex_log("article.log")
     assert not rebuild
     assert error_info.program == "LaTeX"
     assert error_info.src == "./article.tex"
-    assert error_info.message.strip() == (LATEX_LOG1_MESSAGE + MESSAGE_SUFFIX).strip()
+    assert (
+        error_info.message.strip()
+        == (LATEX_LOG1_MESSAGE + MESSAGE_SUFFIX.format(path="article.log")).strip()
+    )
 
 
 LATEX_LOG2 = r"""
@@ -327,12 +400,13 @@ not so much
 """
 
 
-def test_parse_latex_log2():
-    rebuild, error_info = parse_latex_log(io.StringIO(LATEX_LOG2))
+def test_parse_latex_log2(tmpdir):
+    with local_file(LATEX_LOG2, "article.log", tmpdir):
+        rebuild, error_info = parse_latex_log("article.log")
     assert not rebuild
     assert error_info.program == "LaTeX"
     assert error_info.src == "(could not detect source file)"
-    assert error_info.message == DEFAULT_MESSAGE
+    assert error_info.message == DEFAULT_MESSAGE.format(path="article.log")
 
 
 LATEX_LOG3 = r"""
@@ -369,12 +443,16 @@ l.2 \item E
 """
 
 
-def test_parse_latex_log3():
-    rebuild, error_info = parse_latex_log(io.StringIO(LATEX_LOG3))
+def test_parse_latex_log3(tmpdir):
+    with local_file(LATEX_LOG3, "article.log", tmpdir):
+        rebuild, error_info = parse_latex_log("article.log")
     assert not rebuild
     assert error_info.program == "LaTeX"
     assert error_info.src == "./kwart_cirkelboog_e/divergent.inc.tex"
-    assert error_info.message.strip() == (LATEX_LOG3_MESSAGE + MESSAGE_SUFFIX).strip()
+    assert (
+        error_info.message.strip()
+        == (LATEX_LOG3_MESSAGE + MESSAGE_SUFFIX.format(path="article.log")).strip()
+    )
 
 
 LATEX_LOG4 = r"""
@@ -438,12 +516,16 @@ l.116 \end{gather*}
 """
 
 
-def test_parse_latex_log4():
-    rebuild, error_info = parse_latex_log(io.StringIO(LATEX_LOG4))
+def test_parse_latex_log4(tmpdir):
+    with local_file(LATEX_LOG4, "article.log", tmpdir):
+        rebuild, error_info = parse_latex_log("article.log")
     assert not rebuild
     assert error_info.program == "LaTeX"
     assert error_info.src == "./review.tex"
-    assert error_info.message.strip() == (LATEX_LOG4_MESSAGE + MESSAGE_SUFFIX).strip()
+    assert (
+        error_info.message.strip()
+        == (LATEX_LOG4_MESSAGE + MESSAGE_SUFFIX.format(path="article.log")).strip()
+    )
 
 
 LATEX_LOG5 = r"""
@@ -493,24 +575,42 @@ l.40 \end{enumerate}
 """
 
 
-def test_parse_latex_log5():
-    rebuild, error_info = parse_latex_log(io.StringIO(LATEX_LOG5))
+def test_parse_latex_log5(tmpdir):
+    with local_file(LATEX_LOG5, "article.log", tmpdir):
+        rebuild, error_info = parse_latex_log("article.log")
     assert not rebuild
     assert error_info.program == "LaTeX"
     assert error_info.src == "./solutions.tex"
-    assert error_info.message.strip() == (LATEX_LOG5_MESSAGE + MESSAGE_SUFFIX).strip()
+    assert (
+        error_info.message.strip()
+        == (LATEX_LOG5_MESSAGE + MESSAGE_SUFFIX.format(path="article.log")).strip()
+    )
+
+
+def check_source_stack(latex_log, nline, stack, unfinished):
+    lines = latex_log.split("\n")[:nline]
+    lss = LatexSourceStack()
+    for line in lines:
+        lss.feed(line + "\n")
+    assert lss.stack == stack
+    assert lss.unfinished == unfinished
+    assert not lss.unmatched
 
 
 @pytest.mark.parametrize(
     "nline, stack, unfinished",
     [
         (7, ["./solutions.tex"], None),
-        (9, ["./solutions.tex"], "/usr/share/texlive/texmf-dist/tex/latex/base"),
+        (
+            9,
+            ["./solutions.tex"],
+            "L3 programming layer <2022-12-17> (/usr/share/texlive/texmf-dist/tex/latex/base",
+        ),
         (10, ["./solutions.tex", "/usr/share/texlive/texmf-dist/tex/latex/base/article.cls"], None),
         (
             16,
-            ["./solutions.tex", "../../../../preamble.inc.tex"],
-            "/usr/share/texlive/texmf-dist/tex/latex/mathde",
+            ["./solutions.tex", "/usr/share/texlive/texmf-dist/tex/latex/base/article.cls"],
+            ") (../../../../preamble.inc.tex (/usr/share/texlive/texmf-dist/tex/latex/mathde",
         ),
         (
             17,
@@ -526,11 +626,189 @@ def test_parse_latex_log5():
         (30, ["./solutions.tex"], None),
     ],
 )
-def test_latex_source_stack(nline, stack, unfinished):
-    lines = LATEX_LOG5.split("\n")[:nline]
-    lss = LatexSourceStack()
-    for line in lines:
-        lss.feed(line + "\n")
-    assert lss.stack == stack
-    assert lss.unfinished == unfinished
-    assert not lss.unmatched
+def test_latex_source_stack5(nline, stack, unfinished):
+    check_source_stack(LATEX_LOG5, nline, stack, unfinished)
+
+
+LATEX_LOG6 = r"""
+This is XeTeX, Version 3.141592653-2.6-0.999994 (TeX Live 2022/CVE-2023-32700 patched)
+entering extended mode
+ restricted \write18 enabled.
+ %&-line parsing enabled.
+**review
+(./review.tex
+
+[5] (./weerstand_serie_en_parallel2/method.inc.tex) (./weerstand_serie_en_paral
+lel2/answer.inc.tex)
+Overfull \hbox (6.20514pt too wide) in paragraph at lines 315--336
+\T1/mdput/m/n/12 -  []
+ []
+
+(./weerstand_serie_en_parallel2/solution.inc.tex
+File: weerstand_serie_en_parallel2//stappen12.pdf Graphic file (type pdf)
+<use weerstand_serie_en_parallel2//stappen12.pdf>
+[6]
+File: weerstand_serie_en_parallel2//stappen34.pdf Graphic file (type pdf)
+<use weerstand_serie_en_parallel2//stappen34.pdf>
+) (./interactie_eindige_dipolen/stem.inc.tex
+File: interactie_eindige_dipolen//paar_dipolen.pdf Graphic file (type pdf)
+<use interactie_eindige_dipolen//paar_dipolen.pdf>
+) [7] (./interactie_eindige_dipolen/validation.inc.tex)
+! Missing $ inserted.
+<inserted text>
+                $
+l.355
+
+I've inserted something that you may have forgotten.
+(See the <inserted text> above.)
+With luck, this will get me unwedged. But if you
+really didn't forget anything, try typing `2' now; then
+my insertion and my current dilemma will both disappear.
+"""
+
+LATEX_LOG6_MESSAGE = r"""
+! Missing $ inserted.
+<inserted text>
+                $
+l.355
+"""
+
+
+def test_parse_latex_log6(tmpdir):
+    with local_file(LATEX_LOG6, "article.log", tmpdir):
+        rebuild, error_info = parse_latex_log("article.log")
+    assert not rebuild
+    assert error_info.program == "LaTeX"
+    assert error_info.src == "./review.tex"
+    assert (
+        error_info.message.strip()
+        == (LATEX_LOG6_MESSAGE + MESSAGE_SUFFIX.format(path="article.log")).strip()
+    )
+
+
+LATEX_LOG7 = r"""
+(./pdftexcmds.sty
+Package: pdftexcmds 2020-06-27 v0.33 Utility functions of pdfTeX for LuaTeX (HO
+)
+(./infwarerr.sty
+Package: infwarerr 2019/12/03 v1.5 Providing info/warning/error messages (HO)
+)
+)
+"""
+
+
+@pytest.mark.parametrize(
+    "nline, stack, unfinished",
+    [
+        (2, ["./pdftexcmds.sty"], None),
+        (
+            3,
+            ["./pdftexcmds.sty"],
+            "Package: pdftexcmds 2020-06-27 v0.33 Utility functions of pdfTeX for LuaTeX (HO",
+        ),
+        (4, ["./pdftexcmds.sty"], None),
+        (5, ["./pdftexcmds.sty", "./infwarerr.sty"], None),
+        (6, ["./pdftexcmds.sty", "./infwarerr.sty"], None),
+        (7, ["./pdftexcmds.sty"], None),
+        (8, [], None),
+    ],
+)
+def test_latex_source_stack7(nline, stack, unfinished):
+    check_source_stack(LATEX_LOG7, nline, stack, unfinished)
+
+
+LATEX_LOG8 = r"""
+(/usr/share/texlive/texmf-dist/tex/generic/pgf/basiclayer/pgfcorequick.code.tex
+File: pgfcorequick.code.tex 2021/05/15 v3.1.9a (3.1.9a)
+)
+"""
+
+
+@pytest.mark.parametrize(
+    "nline, stack, unfinished",
+    [
+        (
+            3,
+            ["/usr/share/texlive/texmf-dist/tex/generic/pgf/basiclayer/pgfcorequick.code.tex"],
+            None,
+        ),
+    ],
+)
+def test_latex_source_stack8(nline, stack, unfinished):
+    check_source_stack(LATEX_LOG8, nline, stack, unfinished)
+
+
+LATEX_LOG9 = r"""
+(./first.tex
+Bluh
+(./second.tex))
+Blah
+"""
+
+
+@pytest.mark.parametrize(
+    "nline, stack, unfinished",
+    [
+        (2, ["./first.tex"], None),
+        (3, ["./first.tex"], None),
+        (4, [], None),
+        (5, [], None),
+    ],
+)
+def test_latex_source_stack9(nline, stack, unfinished):
+    check_source_stack(LATEX_LOG9, nline, stack, unfinished)
+
+
+SCAN_LATEX_DEPS_EXAMPLE = r"""
+\input{foo.tex}
+%\includegraphics{not.pdf}
+\includegraphics{figure}
+\includegraphics
+{plot.pdf}
+\input{
+    % comments before
+    results/info.tex
+    % comments after
+}
+\input  {
+    % comment 1
+    this
+    % comment 2 }
+    also
+    % comment 3 {
+    works % comment 4
+    % comment 5 }
+}
+%\input{bar.tex}
+\bibliography {references}
+%\bibliography{old}
+\bibliography {
+    extra}
+\import  {sub  % poor formatting
+}    {inc.tex
+}
+%import{sub}{ex.tex}
+"""
+
+
+def test_scan_latex_deps(tmpdir):
+    path_main_tex = os.path.join(tmpdir, "main.tex")
+    with open(path_main_tex, "w") as fh:
+        fh.write(SCAN_LATEX_DEPS_EXAMPLE)
+    implicit, gendeps, bib = scan_latex_deps(path_main_tex, tmpdir)
+    implicit_ref = [
+        "foo.tex",
+        "results/info.tex",
+        "this also works.tex",
+        "figure.pdf",
+        "plot.pdf",
+        "sub/inc.tex",
+    ]
+    implicit_ref = {os.path.join(tmpdir, name) for name in implicit_ref}
+    assert set(implicit) == implicit_ref
+    gendeps_ref = {path for path in implicit_ref if path.endswith(".tex")}
+    gendeps_ref.add(path_main_tex)
+    assert set(gendeps) == gendeps_ref
+    bib_ref = ["references.bib", "extra.bib"]
+    bib_ref = {os.path.join(tmpdir, name) for name in bib_ref}
+    assert set(bib) == bib_ref
